# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/currency.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_currencyWidget(object):
    def setupUi(self, currencyWidget):
        currencyWidget.setObjectName("currencyWidget")
        currencyWidget.resize(383, 169)
        self.gridLayout = QtWidgets.QGridLayout(currencyWidget)
        self.gridLayout.setObjectName("gridLayout")
        self.nameValueLabel = QtWidgets.QLabel(currencyWidget)
        self.nameValueLabel.setText("")
        self.nameValueLabel.setObjectName("nameValueLabel")
        self.gridLayout.addWidget(self.nameValueLabel, 1, 1, 1, 1)
        self.monetaryMassLabel = QtWidgets.QLabel(currencyWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.monetaryMassLabel.setFont(font)
        self.monetaryMassLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.monetaryMassLabel.setObjectName("monetaryMassLabel")
        self.gridLayout.addWidget(self.monetaryMassLabel, 4, 0, 1, 1)
        self.universalDividendValueLabel = QtWidgets.QLabel(currencyWidget)
        self.universalDividendValueLabel.setText("")
        self.universalDividendValueLabel.setObjectName("universalDividendValueLabel")
        self.gridLayout.addWidget(self.universalDividendValueLabel, 3, 1, 1, 1)
        self.universalDividendLabel = QtWidgets.QLabel(currencyWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.universalDividendLabel.setFont(font)
        self.universalDividendLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.universalDividendLabel.setObjectName("universalDividendLabel")
        self.gridLayout.addWidget(self.universalDividendLabel, 3, 0, 1, 1)
        self.membersValueLabel = QtWidgets.QLabel(currencyWidget)
        self.membersValueLabel.setText("")
        self.membersValueLabel.setObjectName("membersValueLabel")
        self.gridLayout.addWidget(self.membersValueLabel, 5, 1, 1, 1)
        self.membersLabel = QtWidgets.QLabel(currencyWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.membersLabel.setFont(font)
        self.membersLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.membersLabel.setObjectName("membersLabel")
        self.gridLayout.addWidget(self.membersLabel, 5, 0, 1, 1)
        self.monetaryMassValueLabel = QtWidgets.QLabel(currencyWidget)
        self.monetaryMassValueLabel.setText("")
        self.monetaryMassValueLabel.setObjectName("monetaryMassValueLabel")
        self.gridLayout.addWidget(self.monetaryMassValueLabel, 4, 1, 1, 1)
        self.nameLabel = QtWidgets.QLabel(currencyWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.nameLabel.setFont(font)
        self.nameLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.nameLabel.setObjectName("nameLabel")
        self.gridLayout.addWidget(self.nameLabel, 1, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 6, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 1, 2, 1, 1)
        self.refreshButton = QtWidgets.QPushButton(currencyWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.refreshButton.sizePolicy().hasHeightForWidth())
        self.refreshButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshButton.setIcon(icon)
        self.refreshButton.setObjectName("refreshButton")
        self.gridLayout.addWidget(self.refreshButton, 0, 2, 1, 1)
        self.symbolLabel = QtWidgets.QLabel(currencyWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.symbolLabel.setFont(font)
        self.symbolLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.symbolLabel.setObjectName("symbolLabel")
        self.gridLayout.addWidget(self.symbolLabel, 2, 0, 1, 1)
        self.symbolValueLabel = QtWidgets.QLabel(currencyWidget)
        self.symbolValueLabel.setText("")
        self.symbolValueLabel.setObjectName("symbolValueLabel")
        self.gridLayout.addWidget(self.symbolValueLabel, 2, 1, 1, 1)

        self.retranslateUi(currencyWidget)
        QtCore.QMetaObject.connectSlotsByName(currencyWidget)

    def retranslateUi(self, currencyWidget):
        #  _translate = QtCore.QCoreApplication.translate
        currencyWidget.setWindowTitle(_("Currency parameters"))
        self.monetaryMassLabel.setText(_("Monetary Mass"))
        self.universalDividendLabel.setText(_("Universal Dividend"))
        self.membersLabel.setText(_("Members"))
        self.nameLabel.setText(_("Name"))
        self.refreshButton.setText(_("Refresh"))
        self.symbolLabel.setText(_("Symbol"))
import tikka.slots.pyqt.resources.icons.index_rc
