# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/main_window.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(843, 605)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setTabsClosable(True)
        self.tabWidget.setMovable(True)
        self.tabWidget.setObjectName("tabWidget")
        self.verticalLayout.addWidget(self.tabWidget)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 843, 25))
        self.menubar.setObjectName("menubar")
        self.accountMenu = QtWidgets.QMenu(self.menubar)
        self.accountMenu.setObjectName("accountMenu")
        self.helpMenu = QtWidgets.QMenu(self.menubar)
        self.helpMenu.setObjectName("helpMenu")
        self.menuNetwork = QtWidgets.QMenu(self.menubar)
        self.menuNetwork.setObjectName("menuNetwork")
        self.menuV1_accounts = QtWidgets.QMenu(self.menubar)
        self.menuV1_accounts.setObjectName("menuV1_accounts")
        self.menuAdvanced = QtWidgets.QMenu(self.menubar)
        self.menuAdvanced.setObjectName("menuAdvanced")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionAccount_tree = QtWidgets.QAction(MainWindow)
        self.actionAccount_tree.setObjectName("actionAccount_tree")
        self.actionAdd_an_address = QtWidgets.QAction(MainWindow)
        self.actionAdd_an_address.setObjectName("actionAdd_an_address")
        self.actionCreate_account = QtWidgets.QAction(MainWindow)
        self.actionCreate_account.setObjectName("actionCreate_account")
        self.actionV1Import_file = QtWidgets.QAction(MainWindow)
        self.actionV1Import_file.setObjectName("actionV1Import_file")
        self.actionQuit = QtWidgets.QAction(MainWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/quit_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionQuit.setIcon(icon)
        self.actionQuit.setObjectName("actionQuit")
        self.actionG1_licence = QtWidgets.QAction(MainWindow)
        self.actionG1_licence.setObjectName("actionG1_licence")
        self.actionConfiguration = QtWidgets.QAction(MainWindow)
        self.actionConfiguration.setObjectName("actionConfiguration")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionConnection = QtWidgets.QAction(MainWindow)
        self.actionConnection.setObjectName("actionConnection")
        self.actionImport_account = QtWidgets.QAction(MainWindow)
        self.actionImport_account.setObjectName("actionImport_account")
        self.actionV1Import_account = QtWidgets.QAction(MainWindow)
        self.actionV1Import_account.setObjectName("actionV1Import_account")
        self.actionCurrency = QtWidgets.QAction(MainWindow)
        self.actionCurrency.setObjectName("actionCurrency")
        self.actionAdd_node = QtWidgets.QAction(MainWindow)
        self.actionAdd_node.setObjectName("actionAdd_node")
        self.actionNodes = QtWidgets.QAction(MainWindow)
        self.actionNodes.setObjectName("actionNodes")
        self.actionAccount_table = QtWidgets.QAction(MainWindow)
        self.actionAccount_table.setObjectName("actionAccount_table")
        self.actionWelcome = QtWidgets.QAction(MainWindow)
        self.actionWelcome.setObjectName("actionWelcome")
        self.actionTransfer = QtWidgets.QAction(MainWindow)
        self.actionTransfer.setObjectName("actionTransfer")
        self.actionImport_in_V2_wizard = QtWidgets.QAction(MainWindow)
        self.actionImport_in_V2_wizard.setObjectName("actionImport_in_V2_wizard")
        self.actionScan_a_QRCode = QtWidgets.QAction(MainWindow)
        self.actionScan_a_QRCode.setObjectName("actionScan_a_QRCode")
        self.actionSmith = QtWidgets.QAction(MainWindow)
        self.actionSmith.setObjectName("actionSmith")
        self.accountMenu.addAction(self.actionTransfer)
        self.accountMenu.addAction(self.actionAccount_tree)
        self.accountMenu.addAction(self.actionAccount_table)
        self.accountMenu.addSeparator()
        self.accountMenu.addAction(self.actionScan_a_QRCode)
        self.accountMenu.addAction(self.actionAdd_an_address)
        self.accountMenu.addAction(self.actionImport_account)
        self.accountMenu.addAction(self.actionCreate_account)
        self.accountMenu.addSeparator()
        self.accountMenu.addAction(self.actionQuit)
        self.helpMenu.addAction(self.actionWelcome)
        self.helpMenu.addAction(self.actionCurrency)
        self.helpMenu.addAction(self.actionG1_licence)
        self.helpMenu.addAction(self.actionConfiguration)
        self.helpMenu.addSeparator()
        self.helpMenu.addAction(self.actionAbout)
        self.menuNetwork.addAction(self.actionConnection)
        self.menuNetwork.addAction(self.actionNodes)
        self.menuNetwork.addAction(self.actionAdd_node)
        self.menuV1_accounts.addAction(self.actionImport_in_V2_wizard)
        self.menuV1_accounts.addAction(self.actionV1Import_account)
        self.menuV1_accounts.addAction(self.actionV1Import_file)
        self.menuAdvanced.addAction(self.actionSmith)
        self.menubar.addAction(self.accountMenu.menuAction())
        self.menubar.addAction(self.menuV1_accounts.menuAction())
        self.menubar.addAction(self.menuNetwork.menuAction())
        self.menubar.addAction(self.menuAdvanced.menuAction())
        self.menubar.addAction(self.helpMenu.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(-1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        #  _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_("MainWindow"))
        self.accountMenu.setTitle(_("Acco&unts"))
        self.helpMenu.setTitle(_("&Help"))
        self.menuNetwork.setTitle(_("Net&work"))
        self.menuV1_accounts.setTitle(_("&V1 accounts"))
        self.menuAdvanced.setTitle(_("Advanc&ed"))
        self.actionAccount_tree.setText(_("Account tr&ee"))
        self.actionAccount_tree.setIconText(_("Account tree"))
        self.actionAdd_an_address.setText(_("A&dd an address"))
        self.actionAdd_an_address.setIconText(_("Add an address"))
        self.actionCreate_account.setText(_("&Create an account"))
        self.actionV1Import_file.setText(_("Import &file"))
        self.actionV1Import_file.setIconText(_("Import file"))
        self.actionQuit.setText(_("&Quit"))
        self.actionG1_licence.setText(_("Ğ1 &licence"))
        self.actionConfiguration.setText(_("&Configuration"))
        self.actionAbout.setText(_("&About"))
        self.actionConnection.setText(_("&Connection"))
        self.actionConnection.setIconText(_("Connection"))
        self.actionImport_account.setText(_("&Import account"))
        self.actionImport_account.setIconText(_("Import account"))
        self.actionV1Import_account.setText(_("Import &account"))
        self.actionV1Import_account.setIconText(_("Import account"))
        self.actionCurrency.setText(_("C&urrency"))
        self.actionAdd_node.setText(_("&Add server"))
        self.actionAdd_node.setIconText(_("Add server"))
        self.actionNodes.setText(_("&Servers"))
        self.actionAccount_table.setText(_("Account ta&ble"))
        self.actionWelcome.setText(_("&Welcome"))
        self.actionTransfer.setText(_("&Transfer"))
        self.actionImport_in_V2_wizard.setText(_("&Import V1 in V2 wizard"))
        self.actionImport_in_V2_wizard.setIconText(_("Import V1 in V2 wizard"))
        self.actionScan_a_QRCode.setText(_("&Scan a QRCode"))
        self.actionSmith.setText(_("&Smith"))
import tikka.slots.pyqt.resources.icons.index_rc
