# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/connection.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ConnectionWidget(object):
    def setupUi(self, ConnectionWidget):
        ConnectionWidget.setObjectName("ConnectionWidget")
        ConnectionWidget.resize(724, 139)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(ConnectionWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.urlsComboBox = QtWidgets.QComboBox(ConnectionWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.urlsComboBox.sizePolicy().hasHeightForWidth())
        self.urlsComboBox.setSizePolicy(sizePolicy)
        self.urlsComboBox.setToolTip("")
        self.urlsComboBox.setEditable(False)
        self.urlsComboBox.setMinimumContentsLength(0)
        self.urlsComboBox.setObjectName("urlsComboBox")
        self.gridLayout.addWidget(self.urlsComboBox, 0, 1, 1, 1)
        self.connectButton = QtWidgets.QPushButton(ConnectionWidget)
        self.connectButton.setObjectName("connectButton")
        self.gridLayout.addWidget(self.connectButton, 0, 2, 1, 1)
        self.connectionStatusLabel = QtWidgets.QLabel(ConnectionWidget)
        self.connectionStatusLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.connectionStatusLabel.setText("")
        self.connectionStatusLabel.setPixmap(QtGui.QPixmap(":/icons/disconnected"))
        self.connectionStatusLabel.setScaledContents(True)
        self.connectionStatusLabel.setObjectName("connectionStatusLabel")
        self.gridLayout.addWidget(self.connectionStatusLabel, 0, 3, 1, 1)
        self.refreshNodeButton = QtWidgets.QPushButton(ConnectionWidget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshNodeButton.setIcon(icon)
        self.refreshNodeButton.setObjectName("refreshNodeButton")
        self.gridLayout.addWidget(self.refreshNodeButton, 0, 4, 1, 1)
        self.versionValueLabel = QtWidgets.QLabel(ConnectionWidget)
        self.versionValueLabel.setText("")
        self.versionValueLabel.setObjectName("versionValueLabel")
        self.gridLayout.addWidget(self.versionValueLabel, 2, 1, 1, 1)
        self.peerIDLabel = QtWidgets.QLabel(ConnectionWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.peerIDLabel.setFont(font)
        self.peerIDLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.peerIDLabel.setObjectName("peerIDLabel")
        self.gridLayout.addWidget(self.peerIDLabel, 3, 0, 1, 1)
        self.serverLabel = QtWidgets.QLabel(ConnectionWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.serverLabel.setFont(font)
        self.serverLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.serverLabel.setObjectName("serverLabel")
        self.gridLayout.addWidget(self.serverLabel, 0, 0, 1, 1)
        self.peerIDValueLabel = QtWidgets.QLabel(ConnectionWidget)
        self.peerIDValueLabel.setText("")
        self.peerIDValueLabel.setObjectName("peerIDValueLabel")
        self.gridLayout.addWidget(self.peerIDValueLabel, 3, 1, 1, 1)
        self.softwareValueLabel = QtWidgets.QLabel(ConnectionWidget)
        self.softwareValueLabel.setText("")
        self.softwareValueLabel.setObjectName("softwareValueLabel")
        self.gridLayout.addWidget(self.softwareValueLabel, 1, 1, 1, 1)
        self.softwareLabel = QtWidgets.QLabel(ConnectionWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.softwareLabel.setFont(font)
        self.softwareLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.softwareLabel.setObjectName("softwareLabel")
        self.gridLayout.addWidget(self.softwareLabel, 1, 0, 1, 1)
        self.versionLabel = QtWidgets.QLabel(ConnectionWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.versionLabel.setFont(font)
        self.versionLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.versionLabel.setObjectName("versionLabel")
        self.gridLayout.addWidget(self.versionLabel, 2, 0, 1, 1)
        self.blockLabel = QtWidgets.QLabel(ConnectionWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.blockLabel.setFont(font)
        self.blockLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.blockLabel.setObjectName("blockLabel")
        self.gridLayout.addWidget(self.blockLabel, 4, 0, 1, 1)
        self.blockValueLabel = QtWidgets.QLabel(ConnectionWidget)
        self.blockValueLabel.setText("")
        self.blockValueLabel.setObjectName("blockValueLabel")
        self.gridLayout.addWidget(self.blockValueLabel, 4, 1, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)

        self.retranslateUi(ConnectionWidget)
        QtCore.QMetaObject.connectSlotsByName(ConnectionWidget)

    def retranslateUi(self, ConnectionWidget):
        #  _translate = QtCore.QCoreApplication.translate
        ConnectionWidget.setWindowTitle(_("Connection"))
        self.connectButton.setToolTip(_("Connection to entry point"))
        self.connectButton.setText(_("Connect"))
        self.refreshNodeButton.setToolTip(_("Refresh nodes from network"))
        self.refreshNodeButton.setText(_("Refresh"))
        self.peerIDLabel.setText(_("Peer ID"))
        self.serverLabel.setText(_("Server"))
        self.softwareLabel.setText(_("Software"))
        self.versionLabel.setText(_("Version"))
        self.blockLabel.setText(_("Block"))
import tikka.slots.pyqt.resources.icons.index_rc
