# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/nodes.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_NodesWidget(object):
    def setupUi(self, NodesWidget):
        NodesWidget.setObjectName("NodesWidget")
        NodesWidget.resize(724, 526)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(NodesWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.addServerButton = QtWidgets.QPushButton(NodesWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.addServerButton.sizePolicy().hasHeightForWidth())
        self.addServerButton.setSizePolicy(sizePolicy)
        self.addServerButton.setObjectName("addServerButton")
        self.verticalLayout_2.addWidget(self.addServerButton, 0, QtCore.Qt.AlignRight)
        self.nodesTableView = QtWidgets.QTableView(NodesWidget)
        self.nodesTableView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.nodesTableView.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.nodesTableView.setShowGrid(True)
        self.nodesTableView.setSortingEnabled(False)
        self.nodesTableView.setObjectName("nodesTableView")
        self.nodesTableView.horizontalHeader().setStretchLastSection(True)
        self.nodesTableView.verticalHeader().setVisible(False)
        self.verticalLayout_2.addWidget(self.nodesTableView)

        self.retranslateUi(NodesWidget)
        QtCore.QMetaObject.connectSlotsByName(NodesWidget)

    def retranslateUi(self, NodesWidget):
        #  _translate = QtCore.QCoreApplication.translate
        NodesWidget.setWindowTitle(_("Network nodes"))
        self.addServerButton.setText(_("Add server"))
import tikka.slots.pyqt.resources.icons.index_rc
